
/* Chrysalide - Outil d'analyse de fichiers binaires
 * rlestr.c - encodage par plage unique d'une chaîne de caractères
 *
 * Copyright (C) 2014-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "rlestr.h"


#include <endian.h>
#include <malloc.h>
#include <sqlite3.h>
#include <string.h>


#include "../../../common/io.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : str  = représentation de chaîne à traiter.                   *
*                data = données à conserver en mémoire.                       *
*                                                                             *
*  Description : Définit une représentation de chaîne de caractères.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void init_rle_string(rle_string *str, const char *data)
{
    if (data != NULL)
    {
        str->data = strdup(data);
        str->length = strlen(data);
    }
    else
    {
        str->data = NULL;
        str->length = 0;
    }

}


/******************************************************************************
*                                                                             *
*  Paramètres  : str  = représentation de chaîne à traiter.                   *
*                data = données à conserver en mémoire.                       *
*                                                                             *
*  Description : Constitue une représentation de chaîne de caractères.        *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void set_rle_string(rle_string *str, const char *data)
{
    if (str->data != NULL)
        unset_rle_string(str);

    if (data != NULL)
    {
        str->data = strdup(data);
        str->length = strlen(data);
    }

}


/******************************************************************************
*                                                                             *
*  Paramètres  : str = représentation de chaîne à traiter.                    *
*                                                                             *
*  Description : Libère la mémoire associée à la représentation.              *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void unset_rle_string(rle_string *str)
{
    if (str->data != NULL)
    {
        free(str->data);
        str->data = NULL;

        str->length = 0;

    }

}


/******************************************************************************
*                                                                             *
*  Paramètres  : s1 = première chaîne à comparer.                             *
*                s2 = seconde chaîne à comparer.                              *
*                                                                             *
*  Description : Effectue la comparaison entre deux chaînes de caractères.    *
*                                                                             *
*  Retour      : Résultat de la comparaison : -1, 0 ou 1.                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

int cmp_rle_string(const rle_string *s1, const rle_string *s2)
{
    int result;                             /* Bilan à retourner           */

    if (s1->length < s2->length)
        result = -1;

    else if (s1->length > s2->length)
        result = 1;

    else
    {
        if (s1->data == NULL && s2->data == NULL)
            result = 0;

        else if (s1->data != NULL && s2->data == NULL)
            result = 1;

        else if (s1->data == NULL && s2->data != NULL)
            result = -1;

        else
            result = strcmp(s1->data, s2->data);

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : str   = informations à constituer. [OUT]                     *
*                fd    = flux ouvert en lecture pour l'importation.           *
*                flags = éventuelles options de réception supplémentaires.    *
*                                                                             *
*  Description : Importe la définition d'une chaîne de caractères.            *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool recv_rle_string(rle_string *str, int fd, int flags)
{
    uint32_t val32;                         /* Valeur sur 32 bits          */
    bool status;                            /* Bilan d'une opération       */

    str->data = NULL;
    str->length = 0;

    status = safe_recv(fd, &val32, sizeof(uint32_t), MSG_WAITALL | flags);
    if (!status) return false;

    str->length = be32toh(val32);

    if (str->length > 0)
    {
        str->data = (char *)malloc(str->length + 1);

        status = safe_recv(fd, str->data, str->length + 1, MSG_WAITALL | flags);
        if (!status)
        {
            unset_rle_string(str);
            return false;
        }

        str->data[str->length] = '\0';

    }

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : str   = informations à sauvegarer.                           *
*                fd    = flux ouvert en écriture pour l'exportation.          *
*                flags = éventuelles options d'envoi supplémentaires.         *
*                                                                             *
*  Description : Exporte la définition d'une chaîne de caractères.            *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool send_rle_string(const rle_string *str, int fd, int flags)
{
    bool status;                            /* Bilan d'une opération       */

    status = safe_send(fd, (uint32_t []) { htobe32(str->length) }, sizeof(uint32_t), MSG_MORE | flags);
    if (!status) return false;

    if (str->length > 0)
    {
        status = safe_send(fd, str->data, str->length + 1, flags);
        if (!status) return false;
    }

    return true;

}



/* ---------------------------------------------------------------------------------- */
/*                       MANIPULATIONS AVEC UNE BASE DE DONNEES                       */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : str    = chaîne de caractères aux informations inutiles.     *
*                name   = désignation personnalisée du champ dans la BD.      *
*                values = couples de champs et de valeurs à lier. [OUT]       *
*                count  = nombre de ces couples. [OUT]                        *
*                                                                             *
*  Description : Constitue les champs destinés à une insertion / modification.*
*                                                                             *
*  Retour      : Bilan de l'opération : succès ou non.                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool prepare_db_statement_for_rle_string(const rle_string *str, const char *name, bound_value **values, size_t *count)
{
    bound_value *value;                     /* Valeur à éditer / définir   */

    *values = (bound_value *)realloc(*values, ++(*count) * sizeof(bound_value));
    value = &(*values)[*count - 1];

    value->cname = name;
    value->built_name = false;
    value->type = (get_rle_string(str) != NULL ? SQLITE_TEXT : SQLITE_NULL);
    value->cstring = get_rle_string(str);
    value->delete = SQLITE_STATIC;

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : str    = chaîne de caractères aux informations inutiles.     *
*                name   = désignation personnalisée du champ dans la BD.      *
*                values = tableau d'éléments à compléter. [OUT]               *
*                count  = nombre de descriptions renseignées. [OUT]           *
*                                                                             *
*  Description : Décrit les colonnes utiles à une chaîne de caractères.       *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool setup_load_of_rle_string(const rle_string *str, const char *name, bound_value **values, size_t *count)
{
    bound_value *value;                     /* Valeur à éditer / définir   */

    *values = (bound_value *)realloc(*values, ++(*count) * sizeof(bound_value));
    value = &(*values)[*count - 1];

    value->cname = name;
    value->built_name = false;
    value->type = SQLITE_NATIVE;

    return true;


}


/******************************************************************************
*                                                                             *
*  Paramètres  : str    = chaîne de caractères à compléter.                   *
*                name   = désignation personnalisée du champ dans la BD.      *
*                values = tableau d'éléments à consulter.                     *
*                count  = nombre de descriptions renseignées.                 *
*                                                                             *
*  Description : Charge les valeurs utiles pour une chaîne de caractères.     *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool load_rle_string(rle_string *str, const char *name, const bound_value *values, size_t count)
{
    const bound_value *value;               /* Valeur à intégrer           */

    value = find_bound_value(values, count, name);
    if (value == NULL) return false;

    switch (value->type)
    {
        case SQLITE_TEXT:
            set_rle_string(str, value->cstring);
            break;

        case SQLITE_NULL:
            set_rle_string(str, NULL);
            break;

        default:
            return false;
            break;

    }

    return true;

}
