
/* Chrysalide - Outil d'analyse de fichiers binaires
 * instruction-int.h - prototypes pour la définition générique interne des instructions
 *
 * Copyright (C) 2008-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ARCH_INSTRUCTION_INT_H
#define _ARCH_INSTRUCTION_INT_H


#include "archbase.h"
#include "instruction.h"



/* Indique l'encodage d'une instruction de façon détaillée. */
typedef const char * (* get_instruction_encoding_fc) (const GArchInstruction *);

/* Fournit le nom humain de l'instruction manipulée. */
typedef const char * (* get_instruction_keyword_fc) (GArchInstruction *, AsmSyntax );

/* Ajoute à un tampon GLib le contenu de l'instance spécifiée. */
typedef GBufferLine * (* print_instruction_fc) (const GArchInstruction *, GBufferLine *, size_t, size_t);

/* Liste les registres lus et écrits par l'instruction. */
typedef void (* get_instruction_rw_regs_fc) (const GArchInstruction *, GArchRegister ***, size_t *, GArchRegister ***, size_t *);



/* Définition générique d'une instruction d'architecture (instance) */
struct _GArchInstruction
{
    GObject parent;                         /* A laisser en premier        */

    phys_t max_displayed_len;               /* Quantité de code affichée   */

    const instr_hook_fc *hooks;             /* Traitements complémentaires */

    mrange_t range;                         /* Emplacement en mémoire      */

    GArchOperand **operands;                /* Liste des opérandes         */
    size_t operands_count;                  /* Nbre. d'opérandes utilisées */

    /**
     * Il existe le besoin indéniable d'un verrou pour les accès aux instructions
     * liées. Il faut par ailleurs un verrou distinct pour les sources et les
     * destination car une même instruction peut boucler sur elle même et la
     * fonction g_arch_instruction_change_link() pose des verrous sur les
     * deux extrémités.
     *
     * Par ailleurs, la consommation mémoire augmente vite : GRWLock pèse 16
     * octets et size_t mesure la taille d'un long sur 64 bits.
     *
     * Que ce soit avec les pointeurs (un allocateur aligne généralement les
     * adresses retournées) ou avec les tailles (il est peu probable d'avoir
     * 0x80000000 instructions d'origine), on dispose d'espace mémoire inutilisé
     * adapté pour compresser la structure GArchInstruction.
     *
     * La GLib propose incidemment des fonctions g_bit_lock() / g_bit_unlock()...
     *
     * On perd au passage la distinction entre les accès en lecture et ceux en
     * écriture, mais tant pis : la réduction de l'empreinte mémoire prime !
     *
     * Par contre la documentation indique :
     *
     * """
     * Attempting to lock on two different bits within the same integer is not supported.
     * """
     *
     * Donc on conserve un compteur distinct pour chaque extrémité.
     */

    instr_link_t *from;                     /* Origines des références     */
    instr_link_t *to;                       /* Instructions visées         */
    gint from_count;                        /* Nombre de ces origines      */
    gint to_count;                          /* Nombre de ces destinations  */

    ArchInstrFlag flags;                    /* Informations complémentaires*/

};


/* Bit de verrou pour les champs (from|to)_count */
#define INSTR_LINK_LOCK_BIT 31


/* Définition générique d'une instruction d'architecture (classe) */
struct _GArchInstructionClass
{
    GObjectClass parent;                    /* A laisser en premier        */

    get_instruction_encoding_fc get_encoding; /* Obtention de l'encodage   */
    get_instruction_keyword_fc get_keyword; /* Texte humain équivalent     */

    print_instruction_fc print;             /* Imprime l'ensemble          */

    //get_instruction_rw_regs_fc get_rw_regs; /* Liste des registres liés    */

};



#endif  /* _ARCH_INSTRUCTION_INT_H */
